#!/bin/bash

##################
# general config #
##################

result_dir=$(pwd)/_out
data_dir=$(pwd)/data

# templates
T_DIR=$(pwd)/templates
T_SIMPLE=$T_DIR/simple
T_SERVER=$T_DIR/server
T_MAKEFILE=$T_DIR/Makefile

# purge first
rm -rf $result_dir

# prepare output structure
[ ! -d $result_dir ] && mkdir -p $result_dir

########################
### generate servers ###
########################

slapd_bin=slapd

# generate 2 servers
for s_num in 1 2; do
    server_dir=$result_dir/server$s_num
    [ ! -d $server_dir ] && mkdir -p $server_dir

    db_dir=$server_dir/db
    run_dir=$server_dir/run
    args_file=$run_dir/slapd.args
    pid_file=$run_dir/slapd.pid
    certdb_dir=$server_dir/certdb
    certs_dir=$server_dir/certs

    mkdir -p $run_dir
    mkdir -p $db_dir

	# generate Makefile
	sed -e "s;@@SLAPD@@;$slapd_bin;" $T_MAKEFILE | tee $server_dir/Makefile

	# server runscript
	runscript=$server_dir/runscript.sh
	# setup cmd
	echo "#!/bin/bash" >> $runscript
	echo "# setup" >> $runscript
	# echo "mkdir -p $slapd_d" >> $runscript
	slapd_d='$1'
	ldif='$2'
	echo "mkdir -p $slapd_d" >> $runscript
	command="slapadd -F $slapd_d -n0 -l $ldif"
	echo $command >> $runscript
	# run cmd
	command="$slapd_bin -F $slapd_d -d9 -h \
	'ldap://ldapserver$s_num.redhat.bug:${s_num}389 \
	ldaps://ldapserver$s_num.redhat.bug:${s_num}636'"

	echo "# run" >> $runscript
	echo $command >> $runscript
	chmod u+x $runscript

	# copy certs
	cp -r $data_dir/certdb $certdb_dir
	cp -r $data_dir/certs $certs_dir

	# servercert - {cbd,pem},replcert
    for s in cdb pem; do
        # server - certdb
        server_cdb="olcTLSCACertificatePath: $certdb_dir\nolcTLSCertificateFile: server"
        # server - pem
        server_pem="olcTLSCACertificateFile: $certs_dir/cacert.pem\n\
olcTLSCertificateFile: $certs_dir/server.pem\n\
olcTLSCertificateKeyFile: $certs_dir/server.pem"
        # repl - certdb
        repl_cdb="tls_cacertdir=\"$certdb_dir\" retry=\"5 +\" timeout=1 tls_cert=\"replicator\""
        # repl - pem
        repl_pem="tls_cacert=\"$certs_dir/cacert.pem\" tls_cacertdir=\"\" retry=\"5 \
+\" timeout=1 tls_cert=\"$certs_dir/replicator.pem\" tls_key=\"$certs_dir/replicatorkey.pem\""

		port1=1389
		port2=1636

        # replcert - {cdb,pem},{cbd,pem}
        for r1 in cdb pem; do
            for r2 in cdb pem; do
                vrs=server_$s
                vr1=repl_$r1
                vr2=repl_$r2
                output=server$s_num-$s-$r1-$r2.ldif

                sed -e "
                    s;@@ARGS_FILE@@;$args_file;
                    s;@@PID_FILE@@;$pid_file;
                    s;@@DB_DIR@@;$db_dir;

                    s;@@SERVER_CERT@@;${!vrs};
                    s;@@REPL_CERT1@@;${!vr1};
                    s;@@REPL_CERT2@@;${!vr2};
                " $T_SERVER | tee $server_dir/$output

                ###################
                # forge runscript #
                ###################

				# runscript=$result_dir/runall.sh
				# subscript=$server_dir/runscript.sh
                # slapd_d="slapd.d-$(basename $output .ldif)"
                # echo "# run" >> $runscript
                # echo "$subscript $slapd_d $output" >> $runscript
				# chmod u+x $runscript





                # runscript=$server_dir/runscript.sh
                # slapd_d="slapd.d-$(basename $output .ldif)"
				# # TODO
				# slapd_d='$1'
				# ldif='$2'
                # # setup cmd
                # echo "# setup" >> $runscript
                # echo "mkdir -p $slapd_d" >> $runscript
                # command="slapadd -F $slapd_d -n0 -l $output"
                # echo $command >> $runscript
                # # run cmd
                # command="$slapd_bin -F $slapd_d -d9 -h \
                # 'ldap://ldapserver$s_num.redhat.bug:$port1 \
                # ldaps://ldapserver$s_num.redhat.bug:$port2 '"
                # echo "# run" >> $runscript
                # echo $command >> $runscript

				# $((port1++))
				# $((port2++))
            done
        done
    done
done

## args, pid, dbdir
# olcArgsFile: /root/bz707599/server1/run/slapd.args
# olcPidFile: /root/bz707599/server1/run/slapd.pid
# olcDbDirectory: /root/bz707599/server1/db

## UP - certdb
#olcTLSCACertificatePath: /root/bz707599/certdb
#olcTLSCertificateFile: server
## UP - pem
#olcTLSCACertificateFile: /root/bz707599/certs/cacert.pem
#olcTLSCertificateFile: /root/bz707599/certs/server.pem
#olcTLSCertificateKeyFile: /root/bz707599/certs/server.pem

## DOWN - certdb
#olcSyncRepl: rid=001 provider=ldaps://ldapserver1.redhat.bug:1636
#  binddn="cn=manager,dc=redhat,dc=bug" bindmethod=sasl
#  saslmech=EXTERNAL searchbase="dc=redhat,dc=bug" type=refreshAndPersist
#  tls_cacertdir="/root/bz707599/certdb" retry="5 +" timeout=1
#  tls_cert="replicator"
#olcSyncRepl: rid=002 provider=ldaps://ldapserver2.redhat.bug:2636
#  binddn="cn=manager,dc=redhat,dc=bug" bindmethod=sasl
#  saslmech=EXTERNAL searchbase="dc=redhat,dc=bug" type=refreshAndPersist
#  tls_cacertdir="/root/bz707599/certdb" retry="5 +" timeout=1
#  tls_cert="replicator"
## DOWN - pem
#olcSyncRepl: rid=001 provider=ldaps://ldapserver1.redhat.bug:1636
#  binddn="cn=manager,dc=redhat,dc=bug" bindmethod=sasl
#  saslmech=EXTERNAL searchbase="dc=redhat,dc=bug" type=refreshAndPersist
#  tls_cacert="/root/bz707599/certs/cacert.pem" tls_cacertdir="" retry="5 +" timeout=1
#  tls_cert="/root/bz707599/certs/replicator.pem"
#  tls_key="/root/bz707599/certs/replicatorkey.pem"
#olcSyncRepl: rid=002 provider=ldaps://ldapserver2.redhat.bug:2636
#  binddn="cn=manager,dc=redhat,dc=bug" bindmethod=sasl
#  saslmech=EXTERNAL searchbase="dc=redhat,dc=bug" type=refreshAndPersist
#  tls_cacert="/root/bz707599/certs/cacert.pem" tls_cacertdir="" retry="5 +" timeout=1
#  tls_cert="/root/bz707599/certs/replicator.pem"
#  tls_key="/root/bz707599/certs/replicatorkey.pem"

###### TODO
### generate run_slapd subscripts for each server
### generate run_test script to test all the combos

